# -*- coding: utf-8 -*-
from . import tr
r"""Transliteration algorythm (TR)

This module provides some operations for replacing or removing
specific characters from source.
"""

VERSION = (0, 1, 2)
__version__ = '0.1.2'
__all__ = ['tr']

tr = tr.tr
